@@ -1,15 +1,37 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 
 from subprocess    import call
 
-from PyQt4.QtCore  import Qt, QObject, SIGNAL, SLOT
-from PyQt4.QtGui   import QAbstractItemView, QCompleter, QDialog, \
+from PyQt4.QtCore  import Qt, QObject, QString, QVariant, SIGNAL, SLOT
+from PyQt4.QtGui   import QAbstractItemView, QComboBox, QDialog, \
                           QDialogButtonBox, QHBoxLayout, QLabel, QLineEdit, \
                           QListWidget, QMenuBar, QMessageBox, QPushButton, \
                           QVBoxLayout, QWidget
 from ACE_file      import ACEFile;
 
-version = '0.0.3'
+version = '0.0.4'
+
+langLookup = {
+    3: 'English (United Kingdom)',
+    4: 'Français (France)',
+    5: 'Deutsch (Deutschland)',
+    6: 'Español (España)',
+    8: 'Português (Portugal)',
+    9: 'Svenska (Sverige)',
+    10: 'Suomi (Suomi)',
+    11: 'Norsk (Norge)',
+    12: 'Dansk (Danmark)',
+    13: 'Nederlands (Nederland)',
+    14: 'Ελληνικά (Ελλάδα)',
+    22: 'Polski (Polska)',
+    23: 'Čeština (Česká republika)',
+    39: 'Русский (Россия)',
+    46: 'English (United States of America)',
+    47: 'Italiano (Italia)',
+    51: 'Español (América Latina)',
+    52: 'Français (Québec)',
+}
 
 class ACEAboutWindow(QDialog): #{{{1
     def __init__(self, parent): #{{{2
@@ -33,8 +55,8 @@ class ACEAboutWindow(QDialog): #{{{1
 
 class ACEMainWindow(QWidget): #{{{1
     _maemo = False
-    _delList = []
-    _addList = []
+    _dict  = None
+    _dicts = {}
 
     def __init__(self): #{{{2
         QWidget.__init__(self, None)
@@ -52,6 +74,11 @@ class ACEMainWindow(QWidget): #{{{1
 
         QHBoxLayout(self)
 
+    def _getCurrentLang(self): #{{{2
+        (lang, check) = self._dictList.itemData(self._dictList.currentIndex()).toInt()
+
+        return lang
+
     def about(self): #{{{2
         aboutWin = ACEAboutWindow(self)
         aboutWin.exec_()
@@ -78,44 +105,58 @@ class ACEMainWindow(QWidget): #{{{1
             return
 
         lwrText = newText.toLower()
+        lang = self._getCurrentLang()
 
-        if lwrText in self._addList:
+        if lwrText in self._dicts[lang]:
             return
 
         self._listView.addItem(lwrText)
+        self._dicts[lang].append(lwrText)
 
-        if lwrText in self._delList:
-            self._delList.removeAt(self._delList.indexOf(lwrText))
-        else:
-            self._addList.append(lwrText)
+    def clearEntries(self, checked): #{{{2
+        lang = self._getCurrentLang()
+        self._listView.clear()
+        self._dicts[lang].clear()
 
     def deleteEntries(self, checked): #{{{2
+        lang = self._getCurrentLang()
+
         for entry in self._listView.selectedItems():
             row = self._listView.row(entry)
             self._listView.takeItem(row)
             val = entry.data(Qt.DisplayRole).toString()
 
-            if val in self._addList:
-                self._addList.removeAt(self._addList.indexOf(val))
-            else:
-                self._delList.append(val)
+            self._dicts[lang].removeAt(self._dicts[lang].indexOf(val))
+
+    def loadDictionary(self, row): #{{{2
+        self._listView.clear()
+
+        if row >= 0:
+            (dictId, check) = self._dictList.itemData(row).toInt()
+            self._listView.addItems(self._dicts[dictId])
 
     def revertChanges(self, checked): #{{{2
-        self._delList = []
-        self._listView.clear()
-        self._listView.addItems(self._dict.dictEntries)
+        self.loadData()
 
     def saveChanges(self, checked): #{{{2
         if self._maemo:
             self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
 
-        for entry in self._delList:
-            pos = self._dict.dictEntries.indexOf(entry)
-            self._dict.dictEntries.removeAt(pos)
+        langs = []
 
-        for entry in self._addList:
-            self._dict.dictEntries.append(entry)
+        for i in self._dicts.keys():
+            if len(self._dicts[i]) > 0:
+                langs.append(i)
 
+        for i in self._dicts.keys():
+            if len(langs) == 0:
+                langs.append(i)
+            
+            if i in langs:
+                self._dict.setDict(i, self._dicts[i])
+            else:
+                self._dict.deleteDict(i)
+            
         self._dict.write()
 
         call(['/usr/bin/killall', 'hildon-input-method'])
@@ -125,6 +166,8 @@ class ACEMainWindow(QWidget): #{{{1
             QMaemo5InformationBox.information(self, 'Dictionary saved', QMaemo5InformationBox.DefaultTimeout)
             self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
 
+        self.loadData()
+
     def setupMenu(self): #{{{2
         menu = QMenuBar()
 
@@ -133,6 +176,10 @@ class ACEMainWindow(QWidget): #{{{1
         self.layout().setMenuBar(menu)
 
     def setupScreen(self): #{{{2
+        self._dictList = QComboBox()
+        self._dictList.setInsertPolicy(QComboBox.NoInsert)
+        QObject.connect(self._dictList, SIGNAL('currentIndexChanged(int)'), self.loadDictionary)
+
         self._listView = QListWidget()
         self._listView.setUniformItemSizes(True)
         self._listView.setSelectionMode(QAbstractItemView.MultiSelection)
@@ -140,6 +187,8 @@ class ACEMainWindow(QWidget): #{{{1
 
         btnBox = QDialogButtonBox()
         btnBox.setOrientation(Qt.Vertical)
+        btn = btnBox.addButton('Delete all', QDialogButtonBox.ActionRole)
+        QObject.connect(btn, SIGNAL('clicked(bool)'), self.clearEntries)
         btn = btnBox.addButton('Delete selected', QDialogButtonBox.ActionRole)
         QObject.connect(btn, SIGNAL('clicked(bool)'), self.deleteEntries)
         btn = btnBox.addButton('Add entry', QDialogButtonBox.ActionRole)
@@ -149,7 +198,11 @@ class ACEMainWindow(QWidget): #{{{1
         btn = btnBox.addButton('Revert changes', QDialogButtonBox.ResetRole)
         QObject.connect(btn, SIGNAL('clicked(bool)'), self.revertChanges)
 
-        self.layout().addWidget(self._listView)
+        lyt = QVBoxLayout()
+        lyt.addWidget(self._dictList)
+        lyt.addWidget(self._listView)
+
+        self.layout().addLayout(lyt)
         self.layout().addWidget(btnBox)
 
         # Clear the progress indicator
@@ -162,16 +215,21 @@ class ACEMainWindow(QWidget): #{{{1
             updateSpinner = True
             self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
 
-        self._dict = ACEFile()
+        if self._dict == None:
+            self._dict = ACEFile()
 
-        if not self._dict.validate():
-            res = QMessageBox.question(self, 'Dictionary file problem', 'The custom auto-complete dictionary does not match a current known format.  Please contact Robin Hill <maemo@robinhill.me.uk> to help get this format supported.  Continuing with editing may break your auto-complete functionality.  Do you wish to continue?', QMessageBox.Yes|QMessageBox.No, QMessageBox.No)
+        dictLangs = self._dict.getLanguages()
+        self._dictList.clear()
+        self._dicts = {}
 
-            if res != QMessageBox.Yes:
-                exit()
+        for lang in dictLangs:
+            self._dicts[lang] = self._dict.getDict(lang)
+            self._dictList.addItem(QString.fromUtf8(langLookup[lang]), QVariant(lang))
 
-        self._listView.addItems(self._dict.dictEntries)
+        self._dictList.setCurrentIndex(0)
 
+        if len(dictLangs) == 1:
+            self._dictList.setEnabled(False)
+        
         if updateSpinner:
             self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
-
